// StandartPSOCPP.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdlib.h>
#include <time.h>
#include <float.h>
#include <math.h>
#include <iostream>

// Computing of test function
float FitnessFunction(float* pos, int D, float min, float max, int function)
{
	// Threatment of particle which leave bounds
	for(int d = 0; d < D; ++d)
	{
		if( (pos[d] < min) || (pos[d] > max) )
		{
			return FLT_MAX;
		}
	}
	
	float return_value = 0;
	switch(function)
	{
	//1st de jong
	case 1:
		{
		for(int d = 0; d < D; d++)
		{
			return_value = return_value + pos[d] * pos[d];
		}
		break;
		}
	// 2nd De Jong
	case 2:
		{
		for(int d = 0; d < D; d++)
		{
			return_value = return_value + (100*(((pos[d]*pos[d])-pos[d+1])*((pos[d]*pos[d])-pos[d+1]))+((1-pos[d])*(1-pos[d])));
		}
		break;
		}
	// 3rd De Jong
	case 3:
		{
		for(int d = 0; d < D; d++)
		{
			return_value = return_value + fabs(pos[d]);
		}
		break;
		}
	default:
		{
			return_value = 999;
			break;
		}
	}
	return return_value;
}

int _tmain(int argc, _TCHAR* argv[])
{
	//Initialization of variables
	int Iter = 300000;		// Number of iterations

	int N = 50;				// Number of particles
	int D = 30;				// Number of dimensions

	float min = -100;		// Bound - minimum allowed value	
	float max = 100;		// Bound - maximum allowed value

	float c1 = 2.05;		// Learning factor
	float c2 = 2.05;		// Learning factor
	float lambda = c1 + c1; // Constant lambda 
	float chi = 2 / ( abs(2 - lambda - sqrt(lambda * lambda - 4 * lambda)) );	// Constant chi
	
	float* X = new float[N * D];	// Particles of the swarm
	float* P = new float[N * D];	// Best positions of each particle
	float* V = new float[N * D];	// Velocity of particles
	float* f = new float[N];		// Fitness of particles
	float times[30];				// Array for calculation times
	float bests[30];				// Array for best fitnesses

	// Variables for measurement of calculation time
	clock_t start, diff;

	// 30 measurement
	for(int y = 0; y < 30; y++)
	{
		srand(time(0));
		// Initialization of the swarm
		for(int i = 0; i < N; ++i)
		{
			f[i] = FLT_MAX;
			for(int d = 0; d < D; ++d)
			{
				// Iterator
				int id = (d + i * D);

				// Initialization of the swarm with random pozition and zero velocity
				X[id] = min + ( (max - min) * (float)rand()/(float)RAND_MAX); 
				V[id] = 0; 
			}
		}

		// Start of measurement of calculation time
		start = clock();

		// PSO algorithm
		for(int k = 0; k < Iter; k++)
		{
			// For every particle
			for(int i = 0; i < N; i++)
			{
				// Pointer to  the current particle
				float* pos = X + i * D; // pozice i-teho jedince v poli
			
				// Fitness calculation of current particle
				float fitness = FitnessFunction( pos, D, min, max, 1);

				// Saving of better fitness and better position of the particle
				if(fitness < f[i])
				{
					f[i] = fitness;
					for(int d = 0; d < D; d++)
					{
						int id = (d + i * D); 
						P[id] = X[id];
					}
				}		
			}

			// For every particle
			for(int i = 0; i < N; i++)
			{
				// Pointer to  the current particle
				float* pos = X + i * D;

				// Pointer to the current best value of particle
				float* pg = pos;

				// Backward particle
				int iB = i - 1; 

				// Forward particle
				int iF = i + 1; // Nasledujici (Forward)

				// Watching and correction of backward and forward particles
				if(i == 0)
				{
					iB = N - 1; 
					iF = 1; 
				} else if( i == (N - 1) )
				{
					iB = N - 2; 
					iF = 0; 
				}

				// Reconfiguration of pointer for best particle if forward or backward particle is better
				if(f[iB] < f[i])
				{
					pg = X + iB * D;

				} else if(f[iF] < f[i])
				{
					pg = X + iF * D;
				}

				// Calculation of new parameters of the current particle
				for(int d = 0; d < D; d++)
				{
					float r1 = (float)rand()/(float)RAND_MAX;
					float r2 = (float)rand()/(float)RAND_MAX;

					int id = (d + i * D); 

					float p1 = c1 * r1 * (P[id] - X[id] );
					float p2 = c2 * r2 * (pg[d] - X[id] );
					V[id] = chi * ( V[id] + p1 + p2); 
					X[id] = X[id] + V[id];
				}
			}
		}
	
		// End of measurement of calculation time
		diff = clock() - start;

		// Printing of calculation time
		int msec = diff * 1000 / CLOCKS_PER_SEC;
		std::cout << "Time taken " << msec/1000 << " seconds and " << msec%1000  << " milliseconds\n" <<std::endl;
		times[y]= (float)msec/1000;

		// Printing of best fitness from calculation
		float bestfit = FLT_MAX; 
		for(int i = 0; i < N; i++)
		{
			if(f[i] < bestfit)
			{
				bestfit = f[i];
			}
		}
		bests[y] = bestfit;
		printf("\nNejlepsi jedinec ma fitness: %f\n",bestfit);
	}

	// Calculation and printing of average fitness and calculation time and their standart deviations 
	float fitmin = FLT_MAX;
	float fitmax = 0;
	float averagetime = 0;
	float averagefit = 0;
	for(int i = 0; i < 30; i++)
	{
		if(bests[i]<fitmin){
			fitmin = bests[i];
		}
		if(bests[i]>fitmax){
			fitmax - bests[i];
		}
		averagetime += times[i];
		averagefit += bests[i];
	}
	averagetime = averagetime/30;
	averagefit = averagefit/30;
	float devtime = 0;
	float devfit = 0;
	for(int i = 0; i < 30; i++)
	{
		devtime += ((times[i]-averagetime)*(times[i]-averagetime));
		devfit += ((bests[i]-averagefit)*(bests[i]-averagefit));
	}
	devtime = devtime/30;
	devtime = sqrt(devtime);
	devfit = devfit/30;
	devfit = sqrt(devfit);
	printf("Average time: %f\nAverage fitness: %f\n Minimum best fitness: %f\n Maximum best fitness: %f", averagetime,averagefit, fitmin, fitmax);
	printf("Standart deviation of time: %f\nStandart deviation of fitness: %f\n ", devtime,devfit);

	// Free memory
	delete[] X;
	delete[] P;	
	delete[] V;	
	delete[] f;	

	getchar();

	return 0;
}

