#pragma OPENCL EXTENSION cl_khr_byte_addressable_store : enable

#define ulong unsigned long

// Pseudorandom number generator
typedef struct 
{ 
	ulong a; 
	ulong b; 
	ulong c; 
	} random_state;

unsigned long random(random_state *r)
{
    unsigned long old = r->b;
    r->b = r->a * 1103515245 + 12345;
    r->a = (~old ^ (r->b >> 3)) - r->c++;
    return r->b;
}

float random_01(random_state *r)
{
    return (random(r) & 4294967295) / 4294967295.0f;
}

void seed_random(random_state *r, ulong seed)
{
    r->a = seed;
    r->b = 0;
    r->c = 362436;
}

// Computing of test function
float test_function( __local float* pos, int D, int function, int min, int max)
{
	// Threatment of particle which leave bounds
	for(int d = 0; d < D; ++d)
	{
		if( (pos[d] < min) || (pos[d] > max) )
		{
			return FLT_MAX;
		}
	}

	float return_value = 0;
	switch(function)
	{
	//1st De Jong
	case 1:
		{
		for(int d = 0; d < D; d++)
		{
			return_value = return_value + pos[d] * pos[d];
		}
		break;
		}
	//2nd De Jong
	case 2:
		{
		for(int d = 0; d < D-1; d++)
		{
			return_value = return_value + (100*(((pos[d]*pos[d])-pos[d+1])*((pos[d]*pos[d])-pos[d+1]))+((1-pos[d])*(1-pos[d])));
		}
		break;
		}
	//3rd De Jong
	case 3:
		{
		for(int d = 0; d < D; d++)
		{
			return_value = return_value + fabs(pos[d]);
		}
		break;
		}
	default:
		{
			return_value = 999;
			break;
		}
	}
	return return_value;
}

// Function to parallel calculation
__kernel void PSO_OpenCL_Kernel
(
	__global float* X, 
	__global float* P, 
	__global float* V, 
	__constant float* parameters, 
	__global float* f,
	__local float* lX
)
{
	// Global and Local ID
	int global_id = get_global_id(0);
	int local_id = get_local_id(0);
	
	// Calculation variables
	int Iter = (int)parameters[0];		// Number of iterations
	int N = (int)parameters[1];			// Number of particles
	int D = (int)parameters[2];		// pocet dimenzi
	float c1 = parameters[3];
	float c2 = parameters[4];
	float chi = parameters[5];
	float min = parameters[6];
	float max = parameters[7];

	//Random number generation
	ulong seeds = (ulong)X[global_id]; 
	random_state r = {0,0,0};
	seed_random(&r, seeds);

	// Copy of values from array X (global memory) to array lX (local memory)
	for(int d = 0; d < D; d++)
	{
		lX[(local_id * D) + d] = X[(global_id * D) + d];
	}

	// Wait until all threads are completed
	barrier(CLK_LOCAL_MEM_FENCE);

	// Pointers to  the current particle
	__local float* x = lX + local_id * D;		// The current coordinate of the current particle in the current dimension
	__global float* p = P + global_id * D;		// The best value of the current particle
	__global float* v = V + global_id * D;		// The current speed of the current particle in the current dimension

	// PSO algorithm
	for(int k = 0; k < Iter; k++)
	{
		// Fitness calculation of current particle
		float fitness = test_function(x, D, 1, min, max);
		
		// Saving of better fitness and better position of the particle
		if(fitness < f[global_id])
		{
			f[global_id] = fitness;
			for(int d = 0; d < D; d++)
			{
				p[d] = x[d];
			}
		}		
	
		// Pointer to the current best value of particle
		__global float* pg = p;

		// Backward particle
		int iB = global_id - 1; 

		// Forward particle
		int iF = global_id + 1; 

		// Watching and correction of backward and forward particles
		if(global_id == 0)
		{
			iB = N - 1; 
			iF = 1; 
		} else if( global_id == (N - 1) )
		{
			iB = N - 2; 
			iF = 0; 
		}

		// Reconfiguration of pointer for best particle if forward or backward particle is better
		if(f[iB] < f[global_id])
		{
			pg = P + iB * D;

		} else if(f[iF] < f[global_id])
		{
			pg = P + iF * D;
		}

		// Calculation of new parameters of the current particle
		for(int d = 0; d < D; d++)
		{
			float r1 = random_01(&r);
			float r2 = random_01(&r);

			float p1 = c1 * r1 * (p[d] -  x[d]);
			float p2 = c2 * r2 * (pg[d] - x[d]);
			v[d] = chi * ( v[d] + p1 + p2 ); 
			x[d] = x[d] + v[d];			
		}
	}
}